/**
* array_name(?TypeName, ?Dim, ?ArrayName)
*
* Gets the array name (eg. String[]) for given type (eg. String) and dimension or
* gets the type and the dimension for given array name.
*/
array_name(TypeName, 0, ArrayName) :-
	nonvar(TypeName),
	ArrayName = TypeName, !.
 
array_name(TypeName, 1, ArrayName) :-
    nonvar(TypeName),
    array_open_brace(OpenBrace),
    array_close_brace(CloseBrace),
    add_braces(TypeName, OpenBrace, CloseBrace, ArrayName), !.

array_name(TypeName, Dim, ArrayName) :-
    nonvar(TypeName),
    nonvar(Dim),
    Dim > 1,
    array_open_brace(OpenBrace),
    array_close_brace(CloseBrace),
    add_braces(TypeName, OpenBrace, CloseBrace, BracedName),
    SubDim is Dim - 1,
    array_name(BracedName, SubDim, ArrayName), !.

array_name(TypeName, Dim, ArrayName) :-
    nonvar(ArrayName),
    array_open_brace(OpenBrace),
    array_close_brace(CloseBrace),
    remove_braces(ArrayName, OpenBrace, CloseBrace, TypeName),
    array_dimension(ArrayName, OpenBrace, Dim), !.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%INTERNAL%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
/**
* array_open_brace(?BraceNumber)
*
* Returns [
*/
array_open_brace(BraceNumber) :-
    atom_codes('[', Matches),
    [BraceNumber | _] = Matches, !.

/**
* array_close_brace(?BraceNumber)
*
* Returns ]
*/
array_close_brace(BraceNumber) :-
    atom_codes(']', Matches),
    [BraceNumber | _] = Matches, !.

/**
* add_braces(+Name, +OpenBrace, +CloseBrace, ?BracedName)
*
* Adds (one pair of) braces (specified by OpenBrace and CloseBrace) to given name
* (eg. String becomes String[]).
*/
add_braces(Name, OpenBrace, CloseBrace, BracedName) :-
    ground([Name, OpenBrace, CloseBrace]),
	string_to_list(Name, Chars),
	append(Chars, [OpenBrace], SingleBraced),
	append(SingleBraced, [CloseBrace], DoubleBraced),
	string_to_list(StringName, DoubleBraced),
	string_to_atom(StringName, BracedName), !.

/**
* remove_braces(+BracedName, +OpenBrace, +CloseBrace, ?Name)
*
* Removes all braces (specified by OpenBrace and CloseBrace) from BracedName
* (eg. String[][] becomes String). 
*/
remove_braces(BracedName, OpenBrace, CloseBrace, Name) :-
    ground([BracedName, OpenBrace, CloseBrace]),
    string_to_list(BracedName, Chars),
    delete(Chars, OpenBrace, SingleBraced),
    delete(SingleBraced, CloseBrace, UnBraced),
    string_to_list(StringName, UnBraced),
    string_to_atom(StringName, Name), !.

/**
* array_dimension(+BracedName, +Brace, ?Dim)
*
* Gets the dimension of given array
* (eg. String[][] has dimension 2).
*/
array_dimension(BracedName, Brace, Dim) :-
    ground([BracedName, Brace]),
    string_to_list(BracedName, BracedChars),
    array_dimension_char(BracedChars, Brace, Dim), !.   

array_dimension_char(BracedChars, Brace, 0) :-
    ground([BracedChars, Brace]),
    not(member(Brace, BracedChars)).

array_dimension_char(BracedChars, Brace, Dim) :-
    ground([BracedChars, Brace]),
    select(Brace, BracedChars, RestChars),
    array_dimension_char(RestChars, Brace, RestDim),
    plus(RestDim, 1, Dim).
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%TEST%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
test(array_name) :-
    assert_true(array_name(int, 0, int)),
    assert_true(array_name(long, 1, 'long[]')),
    assert_true(array_name('String', 4, AN1)),
    assert_true(AN1 == 'String[][][][]'),
    assert_fail(array_name(double, 1, int)),
    assert_true(array_name(T1, D1, 'int[][]')),
    assert_true(T1 == int),
    assert_true(D1 == 2),
    assert_fail(array_name(_, 3, boolean)).

test(add_braces) :-
    array_open_brace(OpenBrace),
    array_close_brace(CloseBrace),
    assert_true(add_braces(int, OpenBrace, CloseBrace, 'int[]')),
    assert_true(add_braces('String[]', OpenBrace, CloseBrace, BracedName)),
    assert_true(BracedName == 'String[][]'),
    assert_fail(add_braces(double, OpenBrace, CloseBrace, 'double[][]')).

test(remove_braces) :-
    array_open_brace(OpenBrace),
    array_close_brace(CloseBrace),
    assert_true(remove_braces('int[]', OpenBrace, CloseBrace, int)),
    assert_true(remove_braces('String[][]', OpenBrace, CloseBrace, Name)),
    assert_true(Name == 'String'),
    assert_fail(remove_braces('double[][][]', OpenBrace, CloseBrace, 'double[]')).

test(array_dimension) :-
    array_open_brace(OpenBrace),
    assert_true(array_dimension('int[]', OpenBrace, 1)),
    assert_true(array_dimension('String[][][]', OpenBrace, Dim)),
    assert_true(Dim  == 3),
    assert_true(array_dimension(double, OpenBrace, 0)),
    assert_fail(array_dimension('float[][]', OpenBrace, 4)).